#setwd() #set Working Directory

######################
## JAGS Model Files ##
######################
write("model{
      # Define the Likelihoods
      for (i0 in 1:N0){
      y0[i0] ~ dnorm(f0[i0],tau.e0)
      f0[i0] <- alpha0 + inprod(X0[i0,],b0[])
      }
      for (i in 1:N){
      y[i] ~ dnorm(f[i],tau.e)
      f[i] <- alpha + inprod(X[i,],b[])
      }
      # Commensurate prior on the intercept
      alpha ~ dnorm(alpha0,eta.alpha)
      eta.alpha <- (1-iota.alpha)*tau.alpha + iota.alpha*R.alpha;
      iota.alpha ~ dbern(p0);
      tau.alpha ~ dunif(sl.alpha,su.alpha);
      # GMC prior on spline coefficients
      b[1] ~ dnorm(mu.b[1],eta.b[1])
      mu.b[1] <- iota.b[1]*b0[1]
      eta.b[1] <- (1-iota.b[1])*slab.b[1] + iota.b[1]*R.b;
      iota.b[1] ~ dbern(nu.b);
      slab.b[1] ~ dunif(sl.b,su.b);
      for(j in 2:J){
      b[j] ~ dnorm(mu.b[j],eta.b[j])
      mu.b[j] <- iota.b[j]*b0[j]
      eta.b[j] <- (1-iota.b[j])*tau.b + iota.b[j]*R.b;
      iota.b[j] ~ dbern(nu.b);
      }
      # Hyperprior on Mixture Weight
      nu.b ~ dbeta(a1,a2);
      # Low-rank thin plate spline prior
      alpha0 ~ dnorm(0,0.0001)
      b0[1] ~ dnorm(0,0.0001)
      for(j in 2:J){
      b0[j] ~ dnorm(0,tau.b0)
      }
      # Hyperpriors on variance parameters
      tau.b <- 1/(sig.b*sig.b)
      sig.b ~ dunif(0.01,100)
      tau.b0 <- 1/(sig.b0*sig.b0)
      sig.b0 ~ dunif(0.01,100)
      tau.e <- 1/(sig.e*sig.e)
      sig.e ~ dunif(0.01,100)
      tau.e0 <- 1/(sig.e0*sig.e0)
      sig.e0 ~ dunif(0.01,100)
      }","GMCModel.txt")

write("model{
      # Define the Likelihood
      for (i in 1:NN){
      yy[i] ~ dnorm(f[i],tau.e)
      f[i] <- alpha + inprod(XX[i,],b[])
      }
      # Low-rank thin plate spline prior
      alpha ~ dnorm(0,0.0001)
      b[1] ~ dnorm(0,0.0001)
      for(j in 2:J){
      b[j] ~ dnorm(0,tau.b)
      }
      # Hyperpriors on variance parameters
      tau.b <- 1/(sig.b*sig.b)
      sig.b ~ dunif(0.01,100)
      tau.e <- 1/(sig.e*sig.e)
      sig.e ~ dunif(0.01,100)
      }","LRTPModel.txt")

###################################
## Simulation Iteration Function ##
###################################
sim = function(N,N0,J,spsl_pars){
  #######################
  ## Generate Datasets ##
  #######################
  d = sample(c(0,0.05,0.1,0.2,0.35,0.5,0.75,1,1.5,2:5),1)
  x = seq(0,1,length.out=N)
  f = function(x,d) (5+d)*x*sin((5+d)*x)
  y = f(x,0) + rnorm(N)
  y0 = f(x,d=d) + rnorm(N0)
  
  #Covariate Range Partition
  tilde.x = 0:J/J
  
  #Transformation/Penalty Matrix
  OMEGA_beta<-rbind(c(1,rep(0,J-1)),cbind(0,abs(outer(tilde.x[-c(1,J+1)],tilde.x[-c(1,J+1)],"-"))^3))
  inv.D_beta<-solve(t(svd(OMEGA_beta)$v%*%(t(svd(OMEGA_beta)$u)*sqrt(svd(OMEGA_beta)$d))))
  #Design Matrix
  X = X0 = cbind(x,t(sapply(x,function(z) abs(z - tilde.x[-c(1,J+1)])^3-tilde.x[-c(1,J+1)]^3)))%*%inv.D_beta
  
  ###################
  ## GMC Model Fit ##
  ###################
  
  #Spike and Slab Hyperparameter Specifications
  sl.alpha = sl.b = spsl_pars[1]
  su.alpha = su.b = spsl_pars[2]
  R.alpha = R.b = spsl_pars[3]
  p0 = a1 = a2 = spsl_pars[4]
  
  #Call JAGS
  dta = list("N", "N0", "y", "y0", "J", "X", "X0", "sl.alpha", "su.alpha", "R.alpha", "p0", "sl.b", "su.b", "R.b", "a1", "a2")
  ints = function() list(sig.e=runif(1,0.01,100), sig.e0=runif(1,0.01,100), sig.b=runif(1,0.01,100), sig.b0=runif(1,0.01,100), b=rnorm(J), b0=rnorm(J), alpha=rnorm(1), alpha0=rnorm(1))
  pars = c("alpha", "b", "alpha0", "b0", "iota.b", "nu.b")
  
  fit <- jags(data=dta,inits=ints,model.file="GMCModel.txt",parameters=pars,n.chains=2,n.iter=52000,n.burnin=2000,n.thin=1)
  
  ######################
  ## Primary Data Fit ##
  ######################
  NN = N; yy = y; XX = X
  dta = list("NN", "yy", "J", "XX")
  ints = function() list(sig.e=runif(1,0.01,100), sig.b=runif(1,0.01,100), b=rnorm(J), alpha=rnorm(1))
  pars = c("alpha","b")
  
  fitI <- jags(data=dta,inits=ints,model.file="LRTPModel.txt",parameters=pars,n.chains=2,n.iter=22000,n.burnin=2000,n.thin=1)
  
  #####################
  ## Pooled Data Fit ##
  #####################
  NN = N+N0; yy = c(y,y0); XX = rbind(X,X)
  dta = list("NN", "yy", "J", "XX")
  ints = function() list(sig.e=runif(1,0.01,100), sig.b=runif(1,0.01,100), b=rnorm(J), alpha=rnorm(1))
  pars = c("alpha","b")
  
  fitP <- jags(data=dta,inits=ints,model.file="LRTPModel.txt",parameters=pars,n.chains=2,n.iter=22000,n.burnin=2000,n.thin=1)
  
  ############################
  ## Compile & Save Results ##
  ############################
  post = apply(fit$BUGSoutput$sims.matrix[,'alpha'] + fit$BUGSoutput$sims.matrix[,sapply(1:J,function(x) paste("b[",x,"]",sep=""))]%*%t(X), 2, function(x) c(mean(x),quantile(x,c(0.025,0.975))))
  postI = apply(fitI$BUGSoutput$sims.matrix[,'alpha'] + fitI$BUGSoutput$sims.matrix[,sapply(1:J,function(x) paste("b[",x,"]",sep=""))]%*%t(X), 2, function(x) c(mean(x),quantile(x,c(0.025,0.975))))
  postP = apply(fitP$BUGSoutput$sims.matrix[,'alpha'] + fitP$BUGSoutput$sims.matrix[,sapply(1:J,function(x) paste("b[",x,"]",sep=""))]%*%t(X), 2, function(x) c(mean(x),quantile(x,c(0.025,0.975))))
  
  ME = c(mean(post[1,]-f(x,d=0)), mean(postI[1,]-f(x,d=0)), mean(postP[1,]-f(x,d=0)))
  RMSE = c(sqrt(mean((post[1,]-f(x,d=0))**2)), sqrt(mean((postI[1,]-f(x,d=0))**2)), sqrt(mean((postP[1,]-f(x,d=0))**2)))
  MCIW = c(mean(post[3,]-post[2,]), mean(postI[3,]-postI[2,]), mean(postP[3,]-postP[2,]))
  MCP = c(mean(ifelse(f(x,d=0) <= post[3,] & f(x,d=0) >= post[2,],1,0)), mean(ifelse(f(x,d=0) <= postI[3,] & f(x,d=0) >= postI[2,],1,0)), mean(ifelse(f(x,d=0) <= postP[3,] & f(x,d=0) >= postP[2,],1,0)))
  
  write.table(rbind(c(d,ME)),"Results/MESimRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
  write.table(rbind(c(d,RMSE)),"Results/RMSESimRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
  write.table(rbind(c(d,MCIW)),"Results/MCIWSimRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
  write.table(rbind(c(d,MCP)),"Results/MCPSimRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
}

################################
## Run Simulation In Parallel ##
################################
library(snowfall)
sfInit(socketHosts=rep("localhost",8),cpus=8,type='SOCK',parallel=TRUE)
sfLibrary(R2jags)
sfExport('sim')
wrapper = function(x){sim(N=50,N0=50,J=10,spsl_pars = c(0,2,2000,.5))}
res = sfLapply(1:2000,wrapper)
sfStop()

##################################
### Compile Simulation Results ###
##################################
MEs = read.table("Results/MESimRes.txt",header=FALSE,sep=",")
RMSEs = read.table("Results/RMSESimRes.txt",header=FALSE,sep=",")
CPs = read.table("Results/MCPSimRes.txt",header=FALSE,sep=",")
CrIWs = read.table("Results/MCIWSimRes.txt",header=FALSE,sep=",")
colnames(MEs) = colnames(RMSEs) = colnames(CPs) = colnames(CrIWs) = c("d","GMC","Curr","Pool")

###############################
### Plot Simulation Results ###
###############################
#Loess Smoothers
d.pred = seq(0,5,by=.01)

par(mfrow=c(2,2),mar=c(4.2,5.2,.2,.2),mgp=c(3, 1, 0),las=1,lab=c(5,5,7))
#MEs
d = MEs[,"d"];y = MEs[,"Curr"];plot(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)),col='red',type='l',ylim=c(-.75,1.25),ylab=expression(ME(d)),xlab=expression(d),lwd=3,lty=2,cex.lab=2,cex.axis=1.5)
y = MEs[,"Pool"];lines(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)),col='blue', lwd=3,lty=4)
y = MEs[,"GMC"];lines(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)), lwd=3)

#RMSEs
d = RMSEs[,"d"];y = RMSEs[,"Curr"];plot(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)),col='red',type='l',ylim=c(0,1.0),ylab=expression(RMSE(d)),xlab=expression(d),lwd=3,lty=2,cex.lab=2,cex.axis=1.5)
y = RMSEs[,"Pool"];lines(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)),col='blue', lwd=3,lty=4)
y = RMSEs[,"GMC"];lines(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)), lwd=3)
legend('topright',c("Primary Alone","Pooled","GMC Prior"),lty=c(2,4,1),lwd=3,col=c("red","blue","black"),bty='n',seg.len=2.5, cex=1.5)

#CPs
d = CPs[,"d"];y = CPs[,"Curr"];plot(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)),col='red',type='l',ylim=c(0.5,1.0),ylab=expression(CP(d)),xlab=expression(d),lwd=3,lty=2,cex.lab=2,cex.axis=1.5)
y = CPs[,"Pool"];lines(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)),col='blue', lwd=3,lty=4)
y = CPs[,"GMC"];lines(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)), lwd=3)
abline(h=0.95,col='grey',lty=3,lwd=2)

#CrIWs
d = CrIWs[,"d"];y = CrIWs[,"Curr"];plot(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)),col='red',type='l',ylim=c(0.5,2),ylab=expression(CrIW(d)),xlab=expression(d),lwd=3,lty=2,cex.lab=2,cex.axis=1.5)
y = CrIWs[,"Pool"];lines(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)),col='blue', lwd=3,lty=4)
y = CrIWs[,"GMC"];lines(d.pred,predict(loess(y ~ d, span=0.4),data.frame(d=d.pred)), lwd=3)
